/* 

mdc2jpeg - a simple converter for files from the MINOLTA RD-175

This program stands under the GNU Public License

(C) Version 1.0 1997 by <beat@rubis.ch> Beat Rubischon

*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <jpeglib.h>

#define CCDSIZE 768 * 494

#ifdef DOS
#define read _read
#define open _open
#endif

int main(int argc, char *argv[]) {

  int fd, ret;
  FILE * jpeg;
  unsigned char *pixelrow;
  int a, b, i, j, f;
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  JSAMPROW row_pointer[1];
  JDIMENSION image_width = 384;
  JDIMENSION image_height = 288;
  char header[512];
  unsigned char *g1 = malloc(CCDSIZE);
  unsigned char *g2 = malloc(CCDSIZE);
  unsigned char *rb = malloc(CCDSIZE);

  if (argc != 4) {
    fprintf(stderr, "\nmdc2jpeg (C) Version 1.0 1997 Beat Rubischon <beat@rubis.ch>\n");
    fprintf(stderr, "Usage: mdc2jpeg 0|1|2 <infile> [<outfile>]\n");
    fprintf(stderr, "           size 0= 384x288, ~25   KB\n");
    fprintf(stderr, "                1= 192x143, ~ 8   KB\n");
    fprintf(stderr, "                2=  96x 71, ~ 2,5 KB\n\n");
    return(-1);
  }

  if (atoi(argv[1]) == 1) {
    image_width = 192;
    image_height = 143;
  }

  if (atoi(argv[1]) == 2) {
    image_width = 96;
    image_height = 71;
  }

  fd = open(argv[2], O_RDONLY);
  if (!fd) {
    fprintf(stderr, "Infile could not be opened\n");
    return(-1);
  }

  ret = read(fd, &header[0], 512);
  if (ret != 512) {
    fprintf(stderr, "Error reading header\n");
    return(-1);
  }

  if (header[0] != 0x00 && header[1] != 0x00) {
    fprintf(stderr, "Wrong magic number\n");
    return(-1);
  }

  for(a = 0; a < 6; a++) {
    for(b = 0; b < 82; b++) {
      if (read(fd, &g1[(b * 6 + a) * 768], 768) != 768) {fprintf(stderr,
        "Error reading CCD data\n"); return(-1); }
    }
    for(b = 0; b < 82; b++) {
      if (read(fd, &rb[(b * 6 + a) * 768], 768) != 768) {fprintf(stderr,
        "Error reading CCD data\n"); return(-1); }
    }
  }

  for(a = 0; a < 6; a++) {
    for(b = 0; b < 82; b++) {
      if (read(fd, &g2[(b * 6 + a) * 768], 768) != 768) {fprintf(stderr,
        "Error reading CCD data\n"); return(-1); }
    }
  }

  if ((jpeg = fopen(argv[3], "wb")) == NULL) {
    fprintf(stderr, "can't open %s\n", argv[3]);
    exit(1);
  }

  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);

  cinfo.image_width = image_width;
  cinfo.image_height = image_height;
  cinfo.input_components = 3;
  cinfo.in_color_space = JCS_RGB;
  jpeg_set_defaults(&cinfo);

  jpeg_set_quality(&cinfo, 75, TRUE /* limit to baseline-JPEG values */);

  jpeg_stdio_dest(&cinfo, jpeg);

  jpeg_start_compress(&cinfo, TRUE);  

  pixelrow = malloc(384 * 3);
  row_pointer[0] = &pixelrow[0];
  f=0;

  if (atoi(argv[1]) == 1) {
    for (i = 0; i < 123; i++) {
      for (j = 0; j < 192; j++) {
        pixelrow[j * 3] = rb[i * 2 * 768 * 2 + j * 2 * 2];
        pixelrow[j * 3 + 1] = g1[i * 2 * 768 * 2 + j * 2 * 2];
        pixelrow[j * 3 + 2] = rb[i * 2 * 768 * 2 + j * 2 * 2 + 1];
      }
      jpeg_write_scanlines(&cinfo, row_pointer, 1);
      f++;
      if (f == 6) {
        f = 0;
        jpeg_write_scanlines(&cinfo, row_pointer, 1);
      }
    }
  }
  else if (atoi(argv[1]) == 2) {
    for (i = 0; i < 61; i++) {
      for (j = 0; j < 96; j++) {
        pixelrow[j * 3] = rb[i * 2 * 768 * 4 + j * 2 * 4];
        pixelrow[j * 3 + 1] = g1[i * 2 * 768 * 4 + j * 2 * 4];
        pixelrow[j * 3 + 2] = rb[i * 2 * 768 * 4 + j * 2 * 4 + 1];
      }
      jpeg_write_scanlines(&cinfo, row_pointer, 1);
      f++;
      if (f == 6) {
        f = 0;
        jpeg_write_scanlines(&cinfo, row_pointer, 1);
      }
    }
  }
  else {
    for (i = 0; i < 247; i++) {
      for (j = 0; j < 384; j++) {
        pixelrow[j * 3] = rb[i * 2 * 768 + j * 2];
        pixelrow[j * 3 + 1] = g1[i * 2 * 768 + j * 2];
        pixelrow[j * 3 + 2] = rb[i * 2 * 768 + j * 2 + 1];
      }
      jpeg_write_scanlines(&cinfo, row_pointer, 1);
      f++;
      if (f == 6) {
        f = 0;
        jpeg_write_scanlines(&cinfo, row_pointer, 1);
      }
    }
  }

  free(pixelrow);
  
  jpeg_finish_compress(&cinfo);

  fclose(jpeg);
  
  jpeg_destroy_compress(&cinfo);
  
  close(fd);
  free(g1);
  free(g2);
  free(rb);

  return(0);
}
